/**********************************************
 * Application: SMS service and demo          *
 * Author: Vladislav Iliev                    *
 * e-mail: vladislav10@abv.bg                 *
 * Year: 2007                                 *
 * FN: M-21557                                *
 * University of Sofia "St. Kliment Ohridski" *
 * Faculty of Mathematics and Informatics     *
 **********************************************/

#include "stdafx.h"
#include "ControlPanel.h"


// static data
ControlPanel* ControlPanel::m_pThis = NULL;

ControlPanel::ControlPanel()
{
    // Set up the static object pointer
    m_pThis = this;
}

ControlPanel::~ControlPanel()
{
}

bool ControlPanel::Install(LPCTSTR name, LPCTSTR binary_path_name)
{
	LPTSTR lpBinaryPathName = new TCHAR[MAX_PATH];
	if (binary_path_name)
		_tcsncpy(lpBinaryPathName, binary_path_name, MAX_PATH);
	else
		::GetModuleFileName(NULL, lpBinaryPathName, MAX_PATH);
	lpBinaryPathName[MAX_PATH - 1] = 0;

	BOOL error = ERROR_SUCCESS;
    HKEY hKey = NULL;
    TCHAR szKey[1024] = _T("Software\\Microsoft\\Windows\\CurrentVersion\\Control Panel\\Cpls\\");
	error = ::RegCreateKeyEx(HKEY_LOCAL_MACHINE, szKey, 0, NULL, 
			REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hKey, NULL);
	if (error == ERROR_SUCCESS)
	{
		::RegSetValueEx(
			hKey,
			name,
			0,
			REG_SZ, 
			(CONST BYTE*)lpBinaryPathName,
			(DWORD)(_tcslen(lpBinaryPathName) + 1) * sizeof(TCHAR)
		);
		RegCloseKey(hKey);
		return true;
	}
	return false;
}

bool ControlPanel::Uninstall(LPCTSTR name)
{
	BOOL error = ERROR_SUCCESS;
    HKEY hKey = NULL;
    TCHAR szKey[1024] = _T("Software\\Microsoft\\Windows\\CurrentVersion\\Control Panel\\Cpls\\");
	error = ::RegCreateKeyEx(HKEY_LOCAL_MACHINE, szKey, 0, NULL, 
			REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hKey, NULL);
	if (error == ERROR_SUCCESS)
	{
		::RegDeleteValue(
			hKey,
			name
		);
		RegCloseKey(hKey);
		return true;
	}
	return false;
}


//////////////////////////////////////////////////////////////////////////////              
// Callback function (exported)

// static member function (callback)
LONG APIENTRY ControlPanel::CPlApplet(
		HWND hwndCPl, 
		UINT uMsg, 
		LPARAM lParam1, 
		LPARAM lParam2)
{
	// Get a pointer to the C++ object
	ControlPanel* pCtrl = m_pThis;

	switch (uMsg) 
	{
		case CPL_DBLCLK:
			return pCtrl->OnDblclk(hwndCPl, (UINT)lParam1, (LONG)lParam2);

		case CPL_EXIT:
			return pCtrl->OnExit();

		case CPL_GETCOUNT:
			return pCtrl->OnGetCount();

		case CPL_INIT:
			return pCtrl->OnInit();

		case CPL_NEWINQUIRE:
			return 0; // not handled

		case CPL_INQUIRE:
			return pCtrl->OnInquire((UINT)lParam1, (CPLINFO*)lParam2);

		case CPL_SELECT:
			return pCtrl->OnSelect((UINT)lParam1, (LONG)lParam2);

		case CPL_STOP:
			return pCtrl->OnStop((UINT)lParam1, (LONG)lParam2);

		default:
			break;
	}

	return 1; // not processed
}

/////////////////////////////////////////////////////////////////////////////////////////
// Default command handlers

LONG ControlPanel::OnDblclk(HWND hwndCPl, UINT uAppNum, LONG lData) 
{
    // Show the dialog
    return 0; // OK
}

LONG ControlPanel::OnExit()
{
    return 0; // OK
}

LONG ControlPanel::OnGetCount()
{
    return 1; // default is support for one dialog box
}

LONG ControlPanel::OnInit()
{
	return 1; // OK
}

LONG ControlPanel::OnSelect(UINT uAppNum, LONG lData) 
{
    return 1; // not handled
}

LONG ControlPanel::OnStop(UINT uAppNum, LONG lData) 
{
    return 1; // not handled
}
