/**********************************************
 * Application: SMS service and demo          *
 * Author: Vladislav Iliev                    *
 * e-mail: vladislav10@abv.bg                 *
 * Year: 2007                                 *
 * FN: M-21557                                *
 * University of Sofia "St. Kliment Ohridski" *
 * Faculty of Mathematics and Informatics     *
 **********************************************/

#pragma once

#include <queue>
#include <list>
#include "cpplib\SerialStream.h"
#include "cpplib\GSM\PDU.h"

class GSM : 
	private	SerialStream
{
private:
	bool m_run; // condition for all thread cycles
	HANDLE m_read; // event controlling reading of unsolicited answers
	HANDLE m_send; // event controlling sending of posted smses

	CRITICAL_SECTION m_send_sec;

	std::list<PDU> m_received;
	std::queue<HANDLE> m_new_sms_events; // blocking each thread waiting for new sms
	SMS m_new_sms; // for transfer of new (incoming) sms to all waiting for it

	std::queue<SMS> m_post_sms;

	CRITICAL_SECTION m_queue_sec; // synchronizes access to m_new_sms_events, m_post_sms

	// settings
	int m_settings_keep_alive_millis;
	// whether to delete new message immediately after its arrival and reading
	bool m_settings_delete_new_sms;
	// how many messages to delete at startup to free enough memory
	int m_settings_delete_at_startup;
	char m_settings_PIN[64];

	static DWORD WINAPI ReadUnsolicited(LPVOID lpThis);
	static DWORD WINAPI SendPosted(LPVOID lpThis);
	
	void StartReadUnsolicited();
	void StopReadUnsolicited();
	void ProcessUnsolicited();
	bool Compose(const char* hexdata);
	// actual (and blocking) sending of sms
	// returns true on success and false on failure
	bool DoSend(SMS sms);
	// 0 - OK
	// >0 - error-code
	// <0 - not a command result
	int ReadCommandResult(int millis);

	void Construct();
	void Destruct();
	void NullInit();
	// (blindly) tests GSM modem capabilities and sets proper 
	// new meassage indication (NMI) settings
	// returns true if such settings exits, otherwise returns false
	bool AutoCNMI();
	// tests GSM modem capabilities and sets proper 
	// prefered message storage (PMS) settings
	bool AutoCPMS();

public:
	GSM();
	~GSM();

	void SetPIN(const char* pin);
	void SetPIN(const wchar_t* pin);
	void SetDeleteNewSMS(bool del);
	void SetDeleteAtStartup(int count);
	void SetKeepAlive(int millis);

	void Connect(LPCTSTR port_name);
	// starts and restarts the modem
	// use it as many times as neccessary - 
	// whenever reconnecting may solve the problem
	void Start();

	void Post(SMS sms);
	bool Send(SMS sms);

	bool Receive(SMS* sms, int millis);
};
