/**********************************************
 * Application: SMS service and demo          *
 * Author: Vladislav Iliev                    *
 * e-mail: vladislav10@abv.bg                 *
 * Year: 2007                                 *
 * FN: M-21557                                *
 * University of Sofia "St. Kliment Ohridski" *
 * Faculty of Mathematics and Informatics     *
 **********************************************/

#pragma once

#include "cpplib\GSM\rpc\sms.h"

class PDU
{
private:
	// SMS-      PDU 
	//     PDU  
	SMS m_sms;
	//  PDU  -  
	char m_units[16][512];
	//   ()   PDU 
	int m_pos[16];
	//   PDU 
	int m_count;
	//    SCA (Service Centre Address)
	int m_scbytes;

protected:
	static int WriteAddress(char* data, const char* address, bool sca);
	static void SetBit(unsigned char arr[], int bit, bool val);
	static int ASCIIToGSM7(const char ascii[], int length, unsigned char gsm7[], int offset);
	static int UnicodeToGSM7(const wchar_t uc[], int length, unsigned char gsm7[], int offset);
	static int ReadTimestamp(const char* hexdata, SMSTimeStamp* ts);
	static bool ParseDeliver(const char* hexdata, SMS& sms, int& part, int& all);
	static int Validity(int minutes);
	void DetectCoding();
	int UnitsUsed() const;

public:
	// constructor
	PDU();

	// for receiving (SMS-DELIVER type)

	//  SMS      PDU
	//  true    PDU   SMS
	//   false
	bool Compose(const char* hexdata);
	//   SMS
	SMS GetSMS() const;

	// for sending (SMS-SUBMIT type)

	//  SMS  units  
	void Decompose(SMS sms);
	//   units  
	int Count() const;
	//     SCA  unit     number
	int UnitServiceCenterBytes() const;
	//    ()  unit     number
	int UnitDataOctets(int number) const;
	//     unit     number
	const char* Unit(int number) const;
};
