/**********************************************
 * Application: SMS service and demo          *
 * Author: Vladislav Iliev                    *
 * e-mail: vladislav10@abv.bg                 *
 * Year: 2007                                 *
 * FN: M-21557                                *
 * University of Sofia "St. Kliment Ohridski" *
 * Faculty of Mathematics and Informatics     *
 **********************************************/

#include "SMSHelper.h"
#include <string.h>

SMSHelper::SMSHelper(void)
{
	memset(this, 0, sizeof(*this));
	Alphabet = -1; // automatic detection
}

SMSHelper::SMSHelper(SMS sms)
: SMS(sms)
{

}

SMSHelper::~SMSHelper(void)
{
}

void SMSHelper::SetServiceCenterAddress(const char* sca)
{
	const int size = sizeof(SCA)/sizeof(SCA[0]);
	strncpy((char*)SCA, sca, size);
	SCA[size-1] = 0;
}

void SMSHelper::SetServiceCenterAddress(const wchar_t* sca)
{
	const int size = sizeof(SCA)/sizeof(SCA[0]);
	WideCharToMultiByte(CP_ACP, 0, sca, -1, (char*)SCA, size, 0, 0);
	SCA[size-1] = 0;
}

void SMSHelper::SetPhone(const char* phone)
{
	const int size = sizeof(Phone)/sizeof(Phone[0]);
	strncpy((char*)Phone, phone, size);
	Phone[size-1] = 0;
}

void SMSHelper::SetPhone(const wchar_t* phone)
{
	const int size = sizeof(Phone)/sizeof(Phone[0]);
	WideCharToMultiByte(CP_ACP, 0, phone, -1, (char*)Phone, size, 0, 0);
	Phone[size-1] = 0;
}

int SMSHelper::GetPhone(wchar_t* phone, int size) const 
{
	int result = MultiByteToWideChar(CP_ACP, 0, (char*)Phone, -1, phone, size);
	if (size > 0)
		phone[size-1] = 0;
	return result;
}

int SMSHelper::GetPhone(char* phone, int size) const
{
	int length = static_cast<int>(strlen((const char*)Phone));
	if (!size)
		return length;
	strncpy(phone, (const char*)Phone, size);
	phone[size-1] = 0;
	if (size-1 < length)
		return size-1;
	return length;
}

void SMSHelper::SetText(const char* text)
{
	const int size = sizeof(Data)/sizeof(Data[0]);
	MultiByteToWideChar(CP_ACP, 0, text, -1, Data, size);
	Data[size-1] = 0;
}

void SMSHelper::SetText(const wchar_t* text)
{
	const int size = sizeof(Data)/sizeof(Data[0]);
	wcsncpy(Data, text, size);
	Data[size-1] = 0;
}
