/**********************************************
 * Application: SMS service and demo          *
 * Author: Vladislav Iliev                    *
 * e-mail: vladislav10@abv.bg                 *
 * Year: 2007                                 *
 * FN: M-21557                                *
 * University of Sofia "St. Kliment Ohridski" *
 * Faculty of Mathematics and Informatics     *
 **********************************************/

#include "stdafx.h"

#include "SQLiteDB.h"
#include <stdio.h>

SQLiteDB::SQLiteDB(void)
{
	m_db = 0;
}

void SQLiteDB::Open(const char* dbname)
{
	Close();
	sqlite3_open(dbname, &m_db);
}

void SQLiteDB::Open(const wchar_t* dbname)
{
	Close();
	sqlite3_open16(dbname, &m_db);
}

void SQLiteDB::Close()
{
	if (m_db)
		sqlite3_close(m_db);
	m_db = 0;
}

SQLiteDB::Statment SQLiteDB::Prepare(const char* query)
{
	Statment s(m_db, query);
	return s;
}

SQLiteDB::Statment SQLiteDB::Prepare(const wchar_t* query)
{
	Statment s(m_db, query);
	return s;
}

int SQLiteDB::GetLastInsertRowId()
{
	return (int)sqlite3_last_insert_rowid(m_db);
}

void SQLiteDB::ErrorMessage(const char** message)
{
	*message = sqlite3_errmsg(m_db);
}

void SQLiteDB::ErrorMessage(const wchar_t** message)
{
	*message = (const wchar_t*)sqlite3_errmsg16(m_db);
}

SQLiteDB::~SQLiteDB(void)
{
	Close();
}

// SQLiteDB::Statment

SQLiteDB::Statment::Statment(sqlite3* db, const char* query)
{
	const char* tail;
	int res = sqlite3_prepare(db, query, -1, &m_stmt, &tail);
	m_refs = new int(0);
	++(*m_refs);
}

SQLiteDB::Statment::Statment(sqlite3* db, const wchar_t* query)
{
	const wchar_t* tail;
	int res = sqlite3_prepare16(db, query, -1, &m_stmt, (const void**) &tail);
	m_refs = new int(0);
	++(*m_refs);
}

SQLiteDB::Statment::Statment(const Statment& right)
{
	m_stmt = right.m_stmt;
	m_refs = right.m_refs;
	++(*m_refs);
}

void SQLiteDB::Statment::operator = (const Statment& right)
{
	sqlite3_stmt* stmt = m_stmt;
	int* refs = m_refs;

	m_stmt = right.m_stmt;
	m_refs = right.m_refs;
	++(*m_refs);

	--(*refs);
	if (!*refs)
	{
		int res = sqlite3_finalize(stmt);
		delete refs;
	}
}

int SQLiteDB::Statment::Step()
{
	return sqlite3_step(m_stmt);
}

void SQLiteDB::Statment::Title(int column, const char** title)
{
	*title = (char*) sqlite3_column_name(m_stmt, column);
}

void SQLiteDB::Statment::Title(int column, const wchar_t** title)
{
	*title = (wchar_t*) sqlite3_column_name16(m_stmt, column);
}

void SQLiteDB::Statment::Text(int column, const char** text)
{
	*text = (char*) sqlite3_column_text(m_stmt, column);
}

void SQLiteDB::Statment::Text(int column, const wchar_t** text)
{
	*text = (wchar_t*) sqlite3_column_text16(m_stmt, column);
}

int SQLiteDB::Statment::Integer(int column)
{
	return sqlite3_column_int(m_stmt, column);
}

SQLiteDB::Statment::~Statment()
{
	--(*m_refs);
	if (!*m_refs)
	{
		int res = sqlite3_finalize(m_stmt);
		delete m_refs;
	}
}

int SQLiteDB::Statment::Columns()
{
	return sqlite3_column_count(m_stmt);
}

