/**********************************************
 * Application: SMS service and demo          *
 * Author: Vladislav Iliev                    *
 * e-mail: vladislav10@abv.bg                 *
 * Year: 2007                                 *
 * FN: M-21557                                *
 * University of Sofia "St. Kliment Ohridski" *
 * Faculty of Mathematics and Informatics     *
 **********************************************/

#include "stdafx.h"
#include "SimpleService.h"

TCHAR SimpleService::m_name[name_length];
SimpleService* SimpleService::m_the_service = 0; 
int SimpleService::m_argument_count;
LPCTSTR* SimpleService::m_argument_values;
SERVICE_STATUS SimpleService::m_service_status; 
SERVICE_STATUS_HANDLE SimpleService::m_service_status_handle;

void SimpleService::OnShutdown()
{
	OnStop();
}

void SimpleService::Run(DWORD argc, LPTSTR *argv)
{
	if (!SetInstance(this))
		return;

	// store arguments in static variables
	m_argument_count = argc;
	if (m_argument_count > 0)
		m_argument_values = new LPCTSTR[m_argument_count];
	for (int i = 0; i < m_argument_count; i++)
		m_argument_values[i] = argv[i];

	SERVICE_TABLE_ENTRY dispatch_table[] = 
	{
		{_T(""), Work}, 
		{NULL, NULL} 
	};
	if (!StartServiceCtrlDispatcher(dispatch_table))
	{
		if (GetLastError() == ERROR_FAILED_SERVICE_CONTROLLER_CONNECT)
		{
			// usually the reason is start like console
			// so behave like console - useful for debugging
			// However, in order to emulate properly you should ensure
			// that service name is set by prior call to SetName
			if (m_the_service->m_name[0] != 0)
			{
				m_the_service->OnStart(argc, argv);
				// stop after the job has been done 
				m_the_service->OnStop();
			}
			else
			{
				// service-name not specified. 
				// use SetName()
			}
		}
	}
	delete[] m_argument_values;
}

int SimpleService::GetArgumentCount()
{
	return m_argument_count;
}

LPCTSTR SimpleService::GetArgumentValue(int n)
{
	return m_argument_values[n];
}

bool SimpleService::SetInstance(SimpleService* svc)
{
	if (m_the_service)
		return false; // error if used twice
	m_the_service = svc;
	return true;
}

void SimpleService::SetName(LPCTSTR name)
{
	_tcsncpy(m_name, name, name_length);
	m_name[name_length - 1] = 0;
}

void WINAPI SimpleService::Work(DWORD comc, LPTSTR *comv) 
{ 
	m_service_status.dwServiceType        = SERVICE_WIN32; 
	m_service_status.dwCurrentState       = SERVICE_START_PENDING; 
	m_service_status.dwControlsAccepted   = SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_SHUTDOWN; 
	m_service_status.dwWin32ExitCode      = 0; 
	m_service_status.dwServiceSpecificExitCode = 0; 
	m_service_status.dwCheckPoint         = 0; 
	m_service_status.dwWaitHint           = 0; 

	m_service_status_handle = RegisterServiceCtrlHandler(_T(""), Handler); 

	if (!m_service_status_handle) 
	{
		return; // error
	}

	m_service_status.dwCurrentState = SERVICE_RUNNING; 
	m_service_status.dwCheckPoint = 0; 
	m_service_status.dwWaitHint = 0; 

	if (!SetServiceStatus(m_service_status_handle, &m_service_status)) 
	{ 
		return; // error
	}

	SetName(comv[0]); // sets the true name given by service manager

	// This is where the service does its work. 
	m_the_service->OnStart(comc, comv);

	return; 
} 

VOID WINAPI SimpleService::Handler(DWORD fdwControl)
{
	switch(fdwControl) 
	{ 
		case SERVICE_CONTROL_STOP: 
			m_the_service->OnStop();

			m_service_status.dwWin32ExitCode = 0; 
			m_service_status.dwCurrentState  = SERVICE_STOPPED; 
			m_service_status.dwCheckPoint    = 0; 
			m_service_status.dwWaitHint      = 0; 
			break;

		case SERVICE_CONTROL_SHUTDOWN: 
			m_the_service->OnShutdown();

			m_service_status.dwWin32ExitCode = 0; 
			m_service_status.dwCurrentState  = SERVICE_STOPPED; 
			m_service_status.dwCheckPoint    = 0; 
			m_service_status.dwWaitHint      = 0; 
			break;

	} 
 
    // Send current status. 
	if (!SetServiceStatus(m_service_status_handle, &m_service_status))
	{
		// error;
	}
}

bool SimpleService::Install(LPCTSTR service_name, LPCTSTR display_name, LPCTSTR binary_path_name, DWORD start_type, LPCTSTR dependencies)
{
	LPCTSTR lpServiceName = service_name;
	LPCTSTR lpDisplayName = display_name;
	DWORD dwStartType = start_type;
	if (display_name == NULL)
	{
		lpDisplayName = service_name;
	}


	LPCTSTR lpMachineName = 0;
	LPCTSTR lpDatabaseName = 0;
	DWORD dwDesiredAccessManager = SC_MANAGER_ALL_ACCESS;
	SC_HANDLE hSCManager = OpenSCManager(
		lpMachineName, 
		lpDatabaseName, 
		dwDesiredAccessManager
	);
	if (hSCManager == NULL)
	{
		return false; // error
	}

	LPTSTR lpBinaryPathName = new TCHAR[MAX_PATH];
	if (binary_path_name)
	{
		_tcsncpy(lpBinaryPathName, binary_path_name, MAX_PATH);
	}
	else
	{
		::GetModuleFileName(NULL, lpBinaryPathName, MAX_PATH);
	}
	lpBinaryPathName[MAX_PATH - 1] = 0;

	DWORD dwDesiredAccessService = SERVICE_ALL_ACCESS;
	DWORD dwServiceType = SERVICE_WIN32_OWN_PROCESS;
	DWORD dwErrorControl = SERVICE_ERROR_NORMAL;
	LPCTSTR lpLoadOrderGroup = 0;
	LPDWORD lpdwTagId = 0;
	LPCTSTR lpDependencies = dependencies;
	LPCTSTR lpServiceStartName = 0;
	LPCTSTR lpPassword = 0;
	SC_HANDLE hSCService = CreateService(
		hSCManager,
		lpServiceName,
		lpDisplayName,
		dwDesiredAccessService,
		dwServiceType,
		dwStartType,
		dwErrorControl,
		lpBinaryPathName,
		lpLoadOrderGroup,
		lpdwTagId,
		lpDependencies,
		lpServiceStartName,
		lpPassword
	);

	DWORD err = 0;
	if (!hSCService)
	{
		err = GetLastError();
	}


	// Event Log
    TCHAR szKey[1024] = _T("SYSTEM\\CurrentControlSet\\Services\\EventLog\\Application\\");
    HKEY hKey = NULL;
    _tcscat(szKey, lpServiceName);
    ::RegCreateKey(HKEY_LOCAL_MACHINE, szKey, &hKey);

    ::RegSetValueEx(hKey,
                    _T("EventMessageFile"),
                    0,
                    REG_EXPAND_SZ, 
                    (CONST BYTE*)lpBinaryPathName,
                    (DWORD)(_tcslen(lpBinaryPathName) + 1) * sizeof(TCHAR));

    DWORD dwData = EVENTLOG_ERROR_TYPE | EVENTLOG_WARNING_TYPE | EVENTLOG_INFORMATION_TYPE;
    ::RegSetValueEx(hKey,
                    _T("TypesSupported"),
                    0,
                    REG_DWORD,
                    (CONST BYTE*)&dwData,
                     sizeof(DWORD));
    ::RegCloseKey(hKey);

	delete[] lpBinaryPathName;

	if (hSCService == NULL)
	{
		CloseServiceHandle(hSCManager);
		return false; // error
	}
	// no checks because nothing to do if the following fail
	CloseServiceHandle(hSCService);	
	CloseServiceHandle(hSCManager);
	return true;
}

bool SimpleService::IsInstalled(LPCTSTR service_name)
{
	bool result = false;
	LPCTSTR lpServiceName = service_name;
	LPCTSTR lpMachineName = 0;
	LPCTSTR lpDatabaseName = 0;
	DWORD dwDesiredAccessManager = SC_MANAGER_ALL_ACCESS;
	SC_HANDLE hSCManager = ::OpenSCManager(
		lpMachineName, 
		lpDatabaseName, 
		dwDesiredAccessManager
	);
	if (hSCManager)
	{
		SC_HANDLE hService = ::OpenService(
			hSCManager, 
			lpServiceName, 
			SERVICE_QUERY_CONFIG
		);
		if (hService)
		{
			result = true;
			::CloseServiceHandle(hService);
		}
		::CloseServiceHandle(hSCManager);
	}
	return result;
}


bool SimpleService::Uninstall(LPCTSTR service_name)
{
	LPCTSTR lpMachineName = 0;
	LPCTSTR lpDatabaseName = 0;
	DWORD dwDesiredAccessManager = SC_MANAGER_ALL_ACCESS;
	SC_HANDLE hSCManager = OpenSCManager(
		lpMachineName, 
		lpDatabaseName, 
		dwDesiredAccessManager
	);
	if (hSCManager == NULL)
	{
		return false; // error
	}

	LPCTSTR lpServiceName = service_name;
	DWORD dwDesiredAccessService = SERVICE_ALL_ACCESS;
	SC_HANDLE hSCService = OpenService(
		hSCManager,
		lpServiceName,
		dwDesiredAccessService
	);
	if (hSCService == NULL)
	{
		CloseServiceHandle(hSCManager);
		return false; // error
	}

	BOOL result;
	result = DeleteService(hSCService);

	// no checks because nothing to do if the following fails
	CloseServiceHandle(hSCService);	
	CloseServiceHandle(hSCManager);

	return result != 0;
}

bool SimpleService::Start(LPCTSTR service_name)
{
	// open the service control manager
	SC_HANDLE hSCM = ::OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
	if (hSCM == NULL)
		return false; // error

	bool success = false;
	// open the service
	SC_HANDLE hService = ::OpenService(hSCM, service_name, SERVICE_ALL_ACCESS);
	if (hService)
	{
		// start the service
		BOOL b = ::StartService(hService, 0, NULL);
		success = (b != FALSE);
		// close the service handle
		::CloseServiceHandle(hService);
	}
	// close the service control manager handle
	::CloseServiceHandle(hSCM);
	return success;
}

bool SimpleService::Stop(LPCTSTR service_name)
{
	// open the service control manager
	SC_HANDLE hSCM = ::OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
	if (hSCM == NULL)
		return false; // error

	bool success = false;
	// open the service
	SC_HANDLE hService = ::OpenService(hSCM, service_name, SERVICE_ALL_ACCESS);
	if (hService)
	{
		// start the service
		SERVICE_STATUS ss;
		BOOL b = ::ControlService(hService, SERVICE_CONTROL_STOP, &ss);
		success = (b != FALSE);
		// close the service handle
		::CloseServiceHandle(hService);
	}
	// close the service control manager handle
	::CloseServiceHandle(hSCM);
	return success;
}

bool SimpleService::Query(LPCTSTR service_name, SERVICE_STATUS* status)
{
	// open the service control manager
	SC_HANDLE hSCM = ::OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
	if (hSCM == NULL)
		return false; // error

	bool success = false;
	// open the service
	SC_HANDLE hService = ::OpenService(hSCM, service_name, SERVICE_ALL_ACCESS);
	if (hService) 
	{
		// Get the current status
		memset(status, 0, sizeof(SERVICE_STATUS));
		BOOL b = ::QueryServiceStatus(hService, status);
		if (b) 
		{
			success = (b != FALSE);
			// If the service is running, send a control request
			// to get its current status
			if (status->dwCurrentState == SERVICE_RUNNING) 
			{
				b = ::ControlService(hService, SERVICE_CONTROL_INTERROGATE, status);
				success = (b != FALSE);
			}
		}
		// close the service handle
		CloseServiceHandle(hService);
	}
	// close the service control manager handle
	CloseServiceHandle(hSCM);
	return success;
}

bool SimpleService::SetParameter(LPCTSTR name, LPCTSTR value)
{
	bool result = false;
	HKEY params = OpenParameters();
	if (params != NULL)
	{
		DWORD length = static_cast<DWORD>(_tcslen(value)) + 1; // 1 for terminating zero
		length *= sizeof(value[0]);
		if (RegSetValueEx(params, name, 0, REG_SZ, 
			(const BYTE*)value, length) == ERROR_SUCCESS)
		{
			result = true;
		}
		CloseParameters(params);
	}
	return result;
}

// TODO: bytes - chars
bool SimpleService::GetParameter(LPCTSTR name, LPCTSTR value, LPDWORD length)
{
	bool result = false;
	HKEY params = OpenParameters();
	if (params != NULL)
	{
		const int elemsize = sizeof(value[0]); // 1 - char, 2 - wchar_t
		DWORD len = 0; // length in bytes
		// determine what len is reqired; value remains unchanged
		if (RegQueryValueEx(params, name, 0, NULL, NULL, &len) == ERROR_SUCCESS)
		{
			// if bytes needed are no more than bytes supplied
			if (len <= *length * elemsize)
			{
				// read the value
				if (RegQueryValueEx(params, name, 0, NULL, (LPBYTE)value, &len) == ERROR_SUCCESS)
				{
					result = true;
				}
			}
		}
		*length = (len + elemsize - 1) / elemsize;
		CloseParameters(params);
	}
	return result;
}

bool SimpleService::SetParameter(LPCTSTR name, DWORD value)
{
	bool result = false;
	HKEY params = OpenParameters();
	if (params != NULL)
	{
		DWORD length = sizeof(value);
		if (RegSetValueEx(params, name, 0, REG_DWORD, 
			(const BYTE*)&value, length) == ERROR_SUCCESS)
		{
			result = true;
		}
		CloseParameters(params);
	}
	return result;
}

bool SimpleService::GetParameter(LPCTSTR name, DWORD& value)
{
	bool result = false;
	HKEY params = OpenParameters();
	if (params != NULL)
	{
		DWORD length = sizeof(value);
		if (RegQueryValueEx(params, name, 0, NULL, 
			(LPBYTE)&value, &length) == ERROR_SUCCESS)
		{
			result = true;
		}
		CloseParameters(params);
	}
	return result;
}

HKEY SimpleService::OpenParameters()
{
	// Parameters are stored under the registry key:
	// HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\<AppName>\Parameters
	HKEY params = NULL;
	TCHAR key[4096];
	_tcscpy(key, _T("SYSTEM\\CurrentControlSet\\Services\\"));
	_tcscat(key, m_name);
	_tcscat(key, _T("\\Parameters"));
	DWORD disp = 0;
	RegCreateKeyEx(
		HKEY_LOCAL_MACHINE, 
		key, 
		0, 
		NULL, 
		REG_OPTION_NON_VOLATILE, 
		KEY_READ | KEY_WRITE, 
		NULL, 
		&params, 
		&disp
	);
	return params;
}

void SimpleService::CloseParameters(HKEY params)
{
	RegCloseKey(params);
}


void SimpleService::LogEvent(WORD wType, DWORD dwID, LPCTSTR info1, LPCTSTR info2, LPCTSTR info3)
{
	LPCTSTR ps[3] = {info1, info2, info3};

	int count = 0;
	for (; count < 3; count++)
		if (ps[count] == NULL) 
			break;

	HANDLE h = RegisterEventSource(NULL, m_name);    // source name 
	if (!h)
		return; // error
        
	ReportEvent(h, wType, 0, dwID, NULL, count, 0, ps, NULL);
	 
	DeregisterEventSource(h); 	
}

