/**********************************************
 * Application: SMS service and demo          *
 * Author: Vladislav Iliev                    *
 * e-mail: vladislav10@abv.bg                 *
 * Year: 2007                                 *
 * FN: M-21557                                *
 * University of Sofia "St. Kliment Ohridski" *
 * Faculty of Mathematics and Informatics     *
 **********************************************/

#pragma once
/* SimpleService class
This class represents simple standalone windows service. "Standalone" here means
that only one instance of class SimpleService is allowed per application. Thus
the compiled execuatble produced by this source can host only one windows 
service. "Simple" here means that only the very basic commands to service are 
used - namely OnStart(), OnStop() and OnShutdown()
Usage: 
1. Create class derivating from SimpleService and define methods OnStart(), 
OnStop() and optionally OnShutdown(). These methods are automatically called 
when Service Control Manages (SCM) issues the corresponding command.
The default behaviour of OnShutdown() is to call OnStop().
2. Create only one object (instance) of the derivate class.
3. Call object's method Run() passing arguments supplied by main(). 
*/

#include <windows.h>
#include <tchar.h>

class SimpleService
{
public:
	/* SimpleService()
	constructor 
	*/
	SimpleService() {};
	/* ~SimpleService()
	virtual destructor allowing derivate classes to execute their own 
	destructor
	*/
	virtual ~SimpleService() {};

	/* Run(DWORD argc, LPTSTR *argv)
	Runs the service
	*/
	void Run(DWORD argc, LPTSTR *argv);

	/* Install(LPCTSTR service_name, LPCTSTR display_name = 0, LPCTSTR binary_path_name = 0, DWORD start_type = SERVICE_AUTO_START)
	Installs new service.
	service_name is the name of the newly created service.
	display_name is user friendly description of the service. If display_name == NULL then
	service_name will be used and for the description.
	binary_path_name is the full path and name of the binary executable that hosts the service
	If left NULL then the path and name of the current module will be used
	start_type - how service is started - automatically (at boot time) or on demand
	*/
	static bool Install(LPCTSTR service_name, LPCTSTR display_name = NULL, LPCTSTR binary_path_name = NULL, DWORD start_type = SERVICE_AUTO_START, LPCTSTR dependencies = NULL);
	static bool IsInstalled(LPCTSTR service_name);
	static bool Uninstall(LPCTSTR service_name);

	static bool Start(LPCTSTR service_name);
	static bool Stop(LPCTSTR service_name);
	static bool Query(LPCTSTR service_name, SERVICE_STATUS* status);

	// the group of functions are valid for calls only after Run() is called
	// or SetName() with proper name as argument
	static bool SetParameter(LPCTSTR name, LPCTSTR value); 
	static bool GetParameter(LPCTSTR name, LPCTSTR value, LPDWORD length);
	static bool SetParameter(LPCTSTR name, DWORD value); 
	static bool GetParameter(LPCTSTR name, DWORD& value);
	static void LogEvent(WORD wType, DWORD dwID, LPCTSTR info1 = NULL, LPCTSTR info2 = NULL, LPCTSTR info3 = NULL);

	static void SetName(LPCTSTR name);

protected:
	/* OnStart(DWORD comc, LPTSTR *comv);
	comc - commands count
	comv - commands values
	Commands are supplied by SCM when issues start command
	comv[0] contains service name
	*/
	virtual void OnStart(DWORD comc, LPTSTR *comv) = 0;
	virtual void OnStop() = 0;
	virtual void OnShutdown();


	static HKEY OpenParameters();
	static void CloseParameters(HKEY params);

	/* Arguments are supplied from the command line used to start the service i.e.
	binPath [arg1] [arg2] [arg3]
	*/
	static int GetArgumentCount();
	static LPCTSTR GetArgumentValue(int n);

private:
	static bool SetInstance(SimpleService* svc);

	static VOID WINAPI Work(DWORD comc, LPTSTR *comv);
	static VOID WINAPI Handler(DWORD fdwControl);

	static const int name_length = 256;
	static TCHAR m_name[name_length];
	static int m_argument_count;
	static LPCTSTR* m_argument_values;
	static SimpleService* m_the_service;
	static SERVICE_STATUS m_service_status; 
	static SERVICE_STATUS_HANDLE m_service_status_handle;
};
