/**********************************************
 * Application: SMS service and demo          *
 * Author: Vladislav Iliev                    *
 * e-mail: vladislav10@abv.bg                 *
 * Year: 2007                                 *
 * FN: M-21557                                *
 * University of Sofia "St. Kliment Ohridski" *
 * Faculty of Mathematics and Informatics     *
 **********************************************/

// PageMonitor.cpp : implementation file
//

#include "stdafx.h"
#include "smsdemo.h"
#include "PageMonitor.h"
#include "cpplib\AutoCritSect.h"
#include "cpplib\GSM\SMSHelper.h"
#include ".\pagemonitor.h"

// OwnerListCtrl

IMPLEMENT_DYNAMIC(OwnerListCtrl, CListCtrl)
OwnerListCtrl::OwnerListCtrl()
{
}

OwnerListCtrl::~OwnerListCtrl()
{
}

BEGIN_MESSAGE_MAP(OwnerListCtrl, CListCtrl)
	ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

BOOL OwnerListCtrl::OnEraseBkgnd(CDC* pDC)
{
	if (GetItemCount() > GetCountPerPage())
	{
		return TRUE; // will be painted later, so don't erase now
	}
	return CListCtrl::OnEraseBkgnd(pDC);
}

void OwnerListCtrl::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct)
{
	CListCtrl& list = *this;
	int item = lpDrawItemStruct->itemID;

	COLORREF bgcol = RGB(255, 255, 255); // UNKNOWN
	COLORREF txcol = RGB(0, 0, 0);

	CString state = list.GetItemText(item, PageMonitor::SMS_STATE);
	if (state == _T("err"))
	{
		bgcol = RGB(255, 0, 0);
		txcol = RGB(255, 255, 255);
	}
	else if (state == _T("snt")) 
	{
		bgcol = RGB(64, 255, 64);
		txcol = RGB(0, 0, 0);
	}
	else if (state == _T("rcv"))
	{
		bgcol = RGB(224, 224, 0);
		txcol = RGB(0, 0, 0);
	}
	else
	{
		return; // UNKNOWN
	}

	CDC* origin = CDC::FromHandle(lpDrawItemStruct->hDC);
	CRect item_rect = lpDrawItemStruct->rcItem;
	
	CDC dcMem;
	dcMem.CreateCompatibleDC(origin);
	CBitmap bm;
	bm.CreateCompatibleBitmap(origin, item_rect.Width(), item_rect.Height());
	dcMem.SelectObject(&bm);

	CDC* pDC = &dcMem;



	bool selected_style = false;
	if (lpDrawItemStruct->itemState & ODS_SELECTED)
	{
		bgcol = RGB(49, 106, 197);// ~bgcol & 0xFFFFFF; //
		txcol = RGB(255, 255, 255);//~txcol & 0xFFFFFF; // 
		selected_style = true;
	}
	CBrush br(bgcol);
	CRect rect = CRect(0, 0, item_rect.Width(), item_rect.Height());
	pDC->FillRect(rect, &br);

	pDC->SetTextColor(txcol);
	pDC->SetBkMode(TRANSPARENT);

	CFont font;
	CFont* old;
	LOGFONT logfont;
	GetFont()->GetLogFont(&logfont);
	if (selected_style)
	{
		logfont.lfItalic = 0;
		logfont.lfUnderline = 0;
		logfont.lfWeight = FW_BOLD;
	}
	font.CreateFontIndirect(&logfont);
	old = pDC->SelectObject(&font);

	for (int col = 0; col < PageMonitor::SMS_END; col++)
	{
		CString text = list.GetItemText(item, col);
		if (col == PageMonitor::SMS_TEXT)
		{
			text.Replace(_T("\r"), _T(" "));
			text.Replace(_T("\n"), _T(" "));
		}
		if (text.GetLength() == 0)
			text = _T(" ");
		CRect subitem_rect;
		list.GetSubItemRect(item, col, LVIR_BOUNDS, subitem_rect);
		subitem_rect -= CPoint(item_rect.left, item_rect.top);
		pDC->DrawText(text, subitem_rect, DT_END_ELLIPSIS);
	}
	font.DeleteObject();
	pDC->SelectObject(old);

	origin->BitBlt(item_rect.left, item_rect.top, 
		item_rect.Width(), item_rect.Height(), &dcMem,
		0, 0, SRCCOPY);
}

// PageMonitor dialog

IMPLEMENT_DYNAMIC(PageMonitor, CPropertyPage)
PageMonitor::PageMonitor()
	: CPropertyPage(PageMonitor::IDD)
	, m_class(0)
{
	InitializeCriticalSection(&m_sect);
	m_edit_contact = new DialogContact();
}

PageMonitor::~PageMonitor()
{
	delete m_edit_contact;
	DeleteCriticalSection(&m_sect);
}

void PageMonitor::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_LIST_SMS, m_messages);
	DDX_Control(pDX, IDC_LIST_CONTACT, m_contacts);
}


BEGIN_MESSAGE_MAP(PageMonitor, CPropertyPage)
	ON_NOTIFY(LVN_ITEMACTIVATE, IDC_LIST_SMS, OnLvnItemActivateMessage)
	ON_NOTIFY(HDN_ITEMCLICK, 0, OnHdnItemclickContact)
	ON_NOTIFY(LVN_ITEMACTIVATE, IDC_LIST_CONTACT, OnLvnItemActivateContact)
	ON_EN_CHANGE(IDC_TEXT, OnEnChangeText)
	ON_BN_CLICKED(IDC_SEND, OnBnClickedSend)
	ON_COMMAND(IDOK, OnBnClickedOk)
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_COMMAND(IDCANCEL, OnBnClickedCancel)
	ON_BN_CLICKED(IDC_BUTTON_DELETE, OnBnClickedButtonDelete)
END_MESSAGE_MAP()


// PageMonitor message handlers

void PageMonitor::RefreshMessages()
{
	CListCtrl* messages = &m_messages;
	messages->DeleteAllItems();

	SQLiteDB* db = CsmsdemoApp::GetDB();
	//                           0,        1,           2,        3,       4,       5
	CString query(_T("SELECT m._id, m._state, m._datetime, m._phone, c._name, m._text FROM message AS m LEFT OUTER JOIN contact AS c ON c._phone = m._phone ORDER BY m._id DESC;"));
	SQLiteDB::Statment stmt = db->Prepare(query);
	int item = 0;
	while(stmt.Step() == SQLITE_ROW)
	{
		messages->InsertItem(item, _T("")); 
		messages->SetItemData(item, stmt.Integer(0));
		const TCHAR* data = 0;
		stmt.Text(1, &data);
		messages->SetItemText(item, SMS_STATE, data);
		stmt.Text(2, &data);
		messages->SetItemText(item, SMS_DATETIME, data);
		stmt.Text(3, &data);
		messages->SetItemText(item, SMS_PHONE, data);
		stmt.Text(4, &data);
		messages->SetItemText(item, SMS_NAME, data);
		stmt.Text(5, &data);
		messages->SetItemText(item, SMS_TEXT, data);
		item++;
	}
	const TCHAR* err = 0;
	db->ErrorMessage(&err);
}

void PageMonitor::RefreshContacts()
{
	m_contacts.DeleteAllItems();

	SQLiteDB* db = CsmsdemoApp::GetDB();
	//                         0,      1,     2
	CString query(_T("SELECT _id, _phone, _name FROM contact"));
	SQLiteDB::Statment stmt = db->Prepare(query);
	int item = 0;
	while(stmt.Step() == SQLITE_ROW)
	{
		m_contacts.InsertItem(item, _T("")); 
		m_contacts.SetItemData(item, stmt.Integer(0));
		const TCHAR* data = 0;
		stmt.Text(1, &data);
		m_contacts.SetItemText(item, CNT_PHONE, data);
		stmt.Text(2, &data);
		m_contacts.SetItemText(item, CNT_NAME, data);
		item++;
	}
}

BOOL PageMonitor::OnInitDialog()
{
	CPropertyPage::OnInitDialog();

	m_edit_contact->Create(m_edit_contact->IDD, this);

	CListCtrl* messages = &m_messages;
	messages->SetExtendedStyle(messages->GetExtendedStyle() | 
		LVS_EX_FULLROWSELECT | LVS_EX_CHECKBOXES);
 
	messages->InsertColumn(SMS_HIDDEN, _T(""), LVCFMT_LEFT, 5);
	messages->InsertColumn(SMS_STATE, _T("S"), LVCFMT_LEFT, 30);
	messages->InsertColumn(SMS_PHONE, _T(""), LVCFMT_LEFT, 100);
	messages->InsertColumn(SMS_DATETIME, _T(", "), LVCFMT_LEFT, 125);
	messages->InsertColumn(SMS_NAME, _T(""), LVCFMT_LEFT, 150);
	messages->InsertColumn(SMS_TEXT, _T(""), LVCFMT_LEFT, 800);

	m_contacts.SetExtendedStyle(m_contacts.GetExtendedStyle() | 
		LVS_EX_FULLROWSELECT | LVS_EX_CHECKBOXES);

	m_contacts.InsertColumn(CNT_SMS, _T("sms"), LVCFMT_LEFT, 40);
	m_contacts.InsertColumn(CNT_PHONE, _T(""), LVCFMT_LEFT, 100);
	m_contacts.InsertColumn(CNT_NAME, _T(""), LVCFMT_LEFT, 250);

	SetDlgItemInt(IDC_EDIT_DELETE, 100);

	RefreshMessages();
	RefreshContacts();
	AddItemNew();

	DWORD threadId;
	CloseHandle(CreateThread(NULL, 0, ReceiverThread, this, 0, &threadId));

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

// --------------------------------------------------------------------------
// This functions works untill it is forcedly terminated by program exit.
// This causes memory leaks but only at end so it is permissible.
// To see if there are any other memory leak - execute without starting
// this function.
DWORD WINAPI PageMonitor::ReceiverThread(LPVOID lpThis)
{
	PageMonitor* self = (PageMonitor*)lpThis;
	if (!self->IsKindOf(RUNTIME_CLASS(PageMonitor)))
		return -1;

	self->Receiver();
	return 0;
}

void PageMonitor::Receiver()
{
	while(true)
	{	
		SMS sms;
		while(true)
		{
			if (CsmsdemoApp::ReceiveSMS(&sms, -1)) // INFINITY
			{
				ProcessSMS(sms);
			}
			else
			{
				// SMS Waiting failed! Will try again in 10 sec
				CString text;
				for (int t = 10; t >= 0; t--)
				{
					text.Format(_T("     ... %d"), t);
					SetDlgItemText(IDC_STATIC_INFO, text);
					Sleep(1000);
				}
				SetDlgItemText(IDC_STATIC_INFO, _T("  RPC  ..."));
				Sleep(5000);
				SetDlgItemText(IDC_STATIC_INFO, _T(""));
			}
		}
	}
}

void PageMonitor::ProcessSMS(SMS sms)
{
	SMSHelper m(sms);
	TCHAR phone[32] = {0};
	m.GetPhone(phone, 32);
	SQLiteDB* db = CsmsdemoApp::GetDB();
	CString query;
	query.Format(_T("INSERT INTO message(_state, _datetime, _phone, _text) VALUES ('%s', '%04d-%02d-%02d %02d:%02d:%02d', '%s', '%s')"),
		_T("rcv"), 
		m.SCTimeStamp.Year, m.SCTimeStamp.Month, m.SCTimeStamp.Day, m.SCTimeStamp.Hour, m.SCTimeStamp.Minute, m.SCTimeStamp.Second, 
		phone, m.Data); // TODO: MBCS
	SQLiteDB::Statment stmt = db->Prepare(query);
	if (stmt.Step() == SQLITE_DONE)
	{
		int id = db->GetLastInsertRowId();
		InsertMessageById(id);
	}
	else
	{
		const TCHAR* err = 0;
		db->ErrorMessage(&err);
	}
}

void PageMonitor::OnLvnItemActivateMessage(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMITEMACTIVATE pNMIA = reinterpret_cast<LPNMITEMACTIVATE>(pNMHDR);
	// To be able to retransmit
	// prepare text for reply - the same from message
	CString text = m_messages.GetItemText(pNMIA->iItem, SMS_TEXT);
	SetDlgItemText(IDC_TEXT, text);
	OnEnChangeText();

	CString phone = m_messages.GetItemText(pNMIA->iItem, SMS_PHONE);
	int count = m_contacts.GetItemCount();
	bool known = false;
	for (int i = 0; i < count; i++)
	{
		if (m_contacts.GetItemText(i, CNT_PHONE) == phone)
		{
			m_contacts.SetCheck(i, TRUE);
			known = true;
		}
		else
		{
			m_contacts.SetCheck(i, FALSE);
		}
	}
	if (!known)
	{
		m_edit_contact->m_exist = false;
		m_edit_contact->m_phone = phone;
		m_edit_contact->m_name = _T("");
		m_edit_contact->m_id = 0;
		
		m_edit_contact->UpdateData(FALSE);
		m_edit_contact->ShowExist();
		m_edit_contact->ShowWindow(SW_SHOW);
	}
	*pResult = 0;
}


void PageMonitor::OnBnClickedOk()
{
//	MessageBox(_T("OK"));
}

void PageMonitor::OnBnClickedCancel()
{
//	MessageBox(_T("Cancel"));
}

void PageMonitor::AddItemNew()
{
	int item = m_contacts.GetItemCount();
	m_contacts.InsertItem(item, _T("")); 
	m_contacts.SetItemData(item, 0);
	m_contacts.SetItemText(item, CNT_PHONE, _T("<<  >>"));
	m_contacts.SetItemText(item, CNT_NAME, _T(""));
}

void PageMonitor::OnBnClickedSend()
{
	CWaitCursor wc;
	int selected = 0;
	int count = m_contacts.GetItemCount();
	for (int item = 0; item < count; item++)
	{
		if (m_contacts.GetCheck(item) && m_contacts.GetItemData(item))
		{
			selected++;
			CString phone = m_contacts.GetItemText(item, CNT_PHONE);
			CString text;
			GetDlgItemText(IDC_TEXT, text);

			SMSHelper hlp;
			hlp.SetPhone(phone);
			hlp.SetText(text);
			BOOL display = ((CButton*)GetDlgItem(IDC_CHECK_DISPLAY))->GetCheck();
			hlp.ClassUse = display != 0;
			if (display)
			{
				hlp.Class = 0;
			}

			BOOL post = ((CButton*)GetDlgItem(IDC_CHECK_POST))->GetCheck();

			CString state = _T("err");
			if (post)
			{
				if (CsmsdemoApp::PostSMS(hlp))
					state = _T("snt");
			}
			else
			{
				if (CsmsdemoApp::SendSMS(hlp))
					state = _T("snt");
			}


			phone.Replace(_T('\''), _T('`'));
			text.Replace(_T('\''), _T('`'));

			SQLiteDB* db = CsmsdemoApp::GetDB();
			CString query;
			SYSTEMTIME st;
			GetLocalTime(&st);
			query.Format(_T("INSERT INTO message(_state, _datetime, _phone, _text) VALUES ('%s', '%04d-%02d-%02d %02d:%02d:%02d', '%s', '%s')"),
				state, st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond, phone, text);
			SQLiteDB::Statment stmt = db->Prepare(query);
			if (stmt.Step() == SQLITE_DONE)
			{
				int id = db->GetLastInsertRowId();
				InsertMessageById(id);
			}
		}
	}
	if (!selected)
	{
		MessageBox(_T("   "), 0, MB_ICONWARNING);
	}
}

void PageMonitor::InsertMessageById(int id)
{
	CString query;
	//                          0,        1,           2,        3,       4,       5
	query.Format(_T("SELECT m._id, m._state, m._datetime, m._phone, c._name, m._text FROM message AS m LEFT OUTER JOIN contact AS c ON c._phone = m._phone WHERE m._id = %d"), id);
	SQLiteDB* db = CsmsdemoApp::GetDB();
	SQLiteDB::Statment stmt = db->Prepare(query);
	int item = 0;
	while(stmt.Step() == SQLITE_ROW)
	{
		m_messages.InsertItem(item, _T("")); 
		m_messages.SetItemData(item, stmt.Integer(0));
		const TCHAR* data = 0;
		stmt.Text(1, &data);
		m_messages.SetItemText(item, SMS_STATE, data);
		stmt.Text(2, &data);
		m_messages.SetItemText(item, SMS_DATETIME, data);
		stmt.Text(3, &data);
		m_messages.SetItemText(item, SMS_PHONE, data);
		stmt.Text(4, &data);
		m_messages.SetItemText(item, SMS_NAME, data);
		stmt.Text(5, &data);
		m_messages.SetItemText(item, SMS_TEXT, data);
		item++;
	}
}

void PageMonitor::OnEnChangeText()
{
	int length = GetDlgItem(IDC_TEXT)->GetWindowTextLength();
	SetDlgItemInt(IDC_LENGTH, length);
}


void PageMonitor::OnLvnItemActivateContact(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMITEMACTIVATE pNMIA = reinterpret_cast<LPNMITEMACTIVATE>(pNMHDR);
	int item = pNMIA->iItem;
	if (item == -1)
		return;
	m_edit_contact->m_id = static_cast<int>(m_contacts.GetItemData(item));
	m_edit_contact->m_exist = m_edit_contact->m_id != 0;
	SQLiteDB* db = CsmsdemoApp::GetDB();
	CString query;
	query.Format(_T("SELECT _id, _phone, _name FROM contact WHERE _id = %d"), m_edit_contact->m_id);
	SQLiteDB::Statment stmt = db->Prepare(query);
	if (stmt.Step() == SQLITE_ROW && m_edit_contact->m_exist)
	{
		const TCHAR* data;
		stmt.Text(1, &data);
		m_edit_contact->m_phone = data;
		stmt.Text(2, &data);
		m_edit_contact->m_name = data;
	}
	else
	{
		m_edit_contact->m_exist = false;
		m_edit_contact->m_phone = _T("359");
		m_edit_contact->m_name = _T("");
	}
	
	m_edit_contact->UpdateData(FALSE);
	m_edit_contact->ShowExist();
	m_edit_contact->ShowWindow(SW_SHOW);

	*pResult = 0;
}

// set all checks if clicked column "sms"
void PageMonitor::OnHdnItemclickContact(NMHDR *pNMHDR, LRESULT *pResult)
{	
	LPNMHEADER phdr = reinterpret_cast<LPNMHEADER>(pNMHDR);
	if (phdr->hdr.hwndFrom == m_contacts.GetHeaderCtrl()->GetSafeHwnd())
	{
		if (phdr->iItem == CNT_SMS)
		{
			int count = m_contacts.GetItemCount();
			bool check = false;
			for (int i = 0; i < count; i++)
			{
				if (!m_contacts.GetCheck(i))
				{
					check = true;
					break;
				}
			}
			for (int i = 0; i < count; i++)
			{
				m_contacts.SetCheck(i, check);
			}
		}
	}
	*pResult = 0;
}

void PageMonitor::OnBnClickedButtonDelete()
{
	int leave = GetDlgItemInt(IDC_EDIT_DELETE);
	CString confirm;
	confirm.Format(_T("  ,       %d ?\n   !"), leave);

	if (MessageBox(confirm, 0, MB_YESNO | MB_DEFBUTTON2) != IDYES)
		return;

	SQLiteDB* db = CsmsdemoApp::GetDB();
	CString query;
	query.Format(_T("DELETE FROM message WHERE _id <= (SELECT MAX(_id)-%d FROM message)"), leave);
	SQLiteDB::Statment stmt = db->Prepare(query);
	if (stmt.Step() == SQLITE_DONE)
	{
		RefreshMessages();
	}
	else
	{
		const TCHAR* err = 0;
		db->ErrorMessage(&err);
	}
}
