/**********************************************
 * Application: SMS service and demo          *
 * Author: Vladislav Iliev                    *
 * e-mail: vladislav10@abv.bg                 *
 * Year: 2007                                 *
 * FN: M-21557                                *
 * University of Sofia "St. Kliment Ohridski" *
 * Faculty of Mathematics and Informatics     *
 **********************************************/

// smsdemo.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "smsdemo.h"
#include "Sheet.h"
#include "cpplib\rpctchar.h"
#include "cpplib\GSM\rpc\sms.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CsmsdemoApp

BEGIN_MESSAGE_MAP(CsmsdemoApp, CWinApp)
	ON_COMMAND(ID_HELP, CWinApp::OnHelp)
END_MESSAGE_MAP()


// CsmsdemoApp construction

CsmsdemoApp::CsmsdemoApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}


// The one and only CsmsdemoApp object

CsmsdemoApp theApp;


// CsmsdemoApp initialization

BOOL CsmsdemoApp::InitInstance()
{
	// InitCommonControls() is required on Windows XP if an application
	// manifest specifies use of ComCtl32.dll version 6 or later to enable
	// visual styles.  Otherwise, any window creation will fail.
	InitCommonControls();

	CWinApp::InitInstance();

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	// of your final executable, you should remove from the following
	// the specific initialization routines you do not need
	// Change the registry key under which our settings are stored
	// TODO: You should modify this string to be something appropriate
	// such as the name of your company or organization
	SetRegistryKey(_T("FMI_M21557"));
	Bind();

	Sheet sheet(_T("SMS Demo"));
	m_pMainWnd = &sheet;
	INT_PTR nResponse = sheet.DoModal();
	if (nResponse == IDOK)
	{
		// TODO: Place code here to handle when the dialog is
		//  dismissed with OK
	}
	else if (nResponse == IDCANCEL)
	{
		// TODO: Place code here to handle when the dialog is
		//  dismissed with Cancel
	}

	// Since the dialog has been closed, return FALSE so that we exit the
	//  application, rather than start the application's message pump.
	return FALSE;
}

SQLiteDB* CsmsdemoApp::GetDB()
{
	static SQLiteDB db;
	static bool open = false;
	if (!open)
	{
		db.Open(_T("sms.db"));

		const char* err = 0;
		SQLiteDB::Statment statement = db.Prepare(_T("CREATE TABLE IF NOT EXISTS contact(_id INTEGER PRIMARY KEY AUTOINCREMENT, _phone TEXT UNIQUE, _name TEXT)"));
		statement.Step();
		db.ErrorMessage(&err);

		statement = db.Prepare(_T("CREATE TABLE IF NOT EXISTS message(_id INTEGER PRIMARY KEY AUTOINCREMENT, _ack INTEGER, _state TEXT, _datetime TEXT, _phone TEXT, _text TEXT)"));
		statement.Step();
		db.ErrorMessage(&err);

		open = true;
	}
	return &db;
}

bool CsmsdemoApp::Bind()
{
	CString protocol_sequence = theApp.GetProfileString(_T("Settings"), _T("protocol_sequence"));
	CString endpoint = theApp.GetProfileString(_T("Settings"), _T("endpoint"));
	CString network_address = theApp.GetProfileString(_T("Settings"), _T("network_address"));

	TCHAR* pszUuid = NULL;
	TCHAR* pszStringBinding = NULL;
	TCHAR* pszOptions = NULL;

	RPC_STATUS status;

	status = RpcStringBindingComposeT(
		pszUuid,
		protocol_sequence.GetBuffer(),
		network_address.GetBuffer(),
		endpoint.GetBuffer(),
		pszOptions,
		&pszStringBinding
	);
	if (status) 
	{
		TCHAR error_message[1024] = {0};
		FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL, (DWORD)status, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
			(LPTSTR) error_message,	1024, NULL);

		MessageBox(0, error_message, _T("Error - RpcStringBindingCompose()"), MB_ICONERROR);
		RpcStringFreeT(&pszStringBinding);
		return false;
	}

	status = RpcBindingFromStringBindingT(
		pszStringBinding,
		&sms_IfHandle
	);
	if (status) 
	{
		TCHAR error_message[1024] = {0};
		FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL, (DWORD)status, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
			(LPTSTR) error_message,	1024, NULL);

		MessageBox(0, error_message, _T("Error - RpcBindingFromStringBinding()"), MB_ICONERROR);
		RpcStringFreeT(&pszStringBinding);
		return false;
	}

	RpcStringFreeT(&pszStringBinding);

	return true;
}

bool CsmsdemoApp::PostSMS(SMS sms)
{
	DWORD ulCode = 0;
	RpcTryExcept  
	{
		Post(sms);
	}
	RpcExcept(1) 
	{
		ulCode = RpcExceptionCode();
		TCHAR error_message[1024] = {0};
		FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL, ulCode, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
			(LPTSTR) error_message,	1024, NULL);

		MessageBox(0, error_message, _T("Error - PostSMS()"), MB_ICONERROR);
	}
	RpcEndExcept
	return ulCode == 0;
}

bool CsmsdemoApp::SendSMS(SMS sms)
{
	DWORD ulCode = 0;
	int result = 0;
	RpcTryExcept  
	{
		result = Send(sms);
	}
	RpcExcept(1) 
	{
		ulCode = RpcExceptionCode();
		TCHAR error_message[1024] = {0};
		FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL, ulCode, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
			(LPTSTR) error_message,	1024, NULL);

		MessageBox(0, error_message, _T("Error - SendSMS()"), MB_ICONERROR);
	}
	RpcEndExcept
	return ulCode == 0 && result != 0;
}

bool CsmsdemoApp::ReceiveSMS(SMS* sms, int timeout)
{
	DWORD ulCode = 0;
	int result = 0;
	RpcTryExcept  
	{
		result = Receive(sms, timeout);
	}
	RpcExcept(1) 
	{
		ulCode = RpcExceptionCode();
	}
	RpcEndExcept
	return ulCode == 0 && result != 0;
}
