/**********************************************
 * Application: SMS service and demo          *
 * Author: Vladislav Iliev                    *
 * e-mail: vladislav10@abv.bg                 *
 * Year: 2007                                 *
 * FN: M-21557                                *
 * University of Sofia "St. Kliment Ohridski" *
 * Faculty of Mathematics and Informatics     *
 **********************************************/

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <windows.h>

#include "cpplib\GSM\rpc\sms.h"
#include "cpplib\rpctchar.h"
#include "SMSService.h"
#include "eventlog.h"

/////////////////////////////////////////////////////////////////////////////
// STATIC VARIABLES
TCHAR SMSService::m_ProtocolSequence[MAX_PARAMLEN] = _T("ncacn_ip_tcp");
TCHAR SMSService::m_Endpoint[MAX_PARAMLEN] = _T("8099");
DWORD SMSService::m_MaxCalls = 20;
DWORD SMSService::m_MaxSize = 65536;
TCHAR SMSService::m_ComPort[MAX_PARAMLEN] = _T("COM2");
TCHAR SMSService::m_PIN[MAX_PARAMLEN] = _T("0000");
DWORD SMSService::m_DeleteNewSMS = 1;
DWORD SMSService::m_DeleteAtStartup = 3;
DWORD SMSService::m_KeepAliveMilliseconds = 10000;


GSM SMSService::m_gsm;
// end of STATIC VARIABLES
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// INITIALIZATION
bool SMSService::InitParameters()
{
	// service parameters

	if (!LoadParameter(_T("RPC_ProtocolSequence"), m_ProtocolSequence)) return false;
	if (!LoadParameter(_T("RPC_Endpoint"), m_Endpoint)) return false;
	if (!LoadParameter(_T("RPC_MaxCalls"), m_MaxCalls)) return false;
	if (!LoadParameter(_T("RPC_MaxSize"), m_MaxSize)) return false;
	if (!LoadParameter(_T("COM_Port"), m_ComPort)) return false;
	if (!LoadParameter(_T("GSM_PINCode"), m_PIN)) return false;
	if (!LoadParameter(_T("GSM_DeleteNewSMS"), m_DeleteNewSMS)) return false;
	if (!LoadParameter(_T("GSM_DeleteAtStartup"), m_DeleteAtStartup)) return false;
	if (!LoadParameter(_T("GSM_KeepAliveMilliseconds"), m_KeepAliveMilliseconds)) return false;

	return true;
}
bool SMSService::LoadParameter(LPCTSTR name, LPTSTR value)
{
	DWORD length = MAX_PARAMLEN;
	TCHAR str_max[64];
	_stprintf(str_max, _T("%d"), MAX_PARAMLEN);

	if (!GetParameter(name, value, &length))
	{
		if (length > MAX_PARAMLEN)
		{
			SimpleService::LogEvent(EVENTLOG_ERROR_TYPE, MSG_PARAMETER_TOO_LONG, 
				name, str_max);
			return false; // error
		}
		// set and use default
		SimpleService::SetParameter(name, value);
	}
	return true; // OK
}
bool SMSService::LoadParameter(LPCTSTR name, DWORD& value)
{
	if (!GetParameter(name, value))
	{
		SimpleService::SetParameter(name, value);
	}
	return true;
}
void SMSService::InitGSM()
{
	m_gsm.SetPIN(m_PIN);
	m_gsm.SetDeleteNewSMS(m_DeleteNewSMS != 0);
	m_gsm.SetDeleteAtStartup(m_DeleteAtStartup);
	m_gsm.SetKeepAlive(m_KeepAliveMilliseconds);
	try
	{
		m_gsm.Connect(m_ComPort);
	}
	catch(PException )
	{
		LogEvent(EVENTLOG_WARNING_TYPE, MSG_COMMON_INFO3, 
			_T("First failure to connect and read/write to GSM.\nIs it turned on and connected to port "), 
			m_ComPort, 
			_T(" ?\nAttempts to connect will continue periodically"));
	}
	try
	{
		m_gsm.Start();
	}
	catch(PException )
	{
	}
}
bool SMSService::InitRPC()
{
	// hardcoded parameters
	TCHAR * pszSecurity = NULL; 
	DWORD   cMinCalls   = 1;
	DWORD   fDontWait   = FALSE;
	
	RPC_STATUS status;

	status = RpcServerUseProtseqEpT(
		m_ProtocolSequence,
		m_MaxCalls,
		m_Endpoint,
		pszSecurity
	); 

	if (status) 
	{
		TCHAR error_message[1024] = {0};
		FormatErrorMessage(status, error_message, 1024);
		LogEvent(EVENTLOG_ERROR_TYPE, MSG_FUNCTION_FAILED, 
			_T("RpcServerUseProtseqEp()"), error_message);
		return false;
	}
	
	// the following (in comments) gives "Access Denied" to clients
	//status = RpcServerRegisterIf(
	//	smsif_v2_0_s_ifspec,  
	//	NULL,   
	//	NULL
	//);
	// so it is replaced by:
	status = RpcServerRegisterIf2(
		sms_v2_0_s_ifspec, 
		NULL, 
		NULL, 
		RPC_IF_ALLOW_CALLBACKS_WITH_NO_AUTH, 
		m_MaxCalls, 
		m_MaxSize, 
		NULL
	);
	if (status) 
	{
		TCHAR error_message[1024] = {0};
		FormatErrorMessage(status, error_message, 1024);
		LogEvent(EVENTLOG_ERROR_TYPE, MSG_FUNCTION_FAILED, 
			_T("RpcServerUseProtseqEp()"), error_message);
		return false;
	}

	status = RpcServerListen(
		cMinCalls,
		RPC_C_LISTEN_MAX_CALLS_DEFAULT,
		fDontWait
	);
	if (status) 
	{
		TCHAR error_message[1024] = {0};
		FormatErrorMessage(status, error_message, 1024);
		LogEvent(EVENTLOG_ERROR_TYPE, MSG_FUNCTION_FAILED, 
			_T("RpcServerListen()"), error_message);
		return false;
	}

	return true;
}
// end of INITIALIZATION
/////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////
// BEHAVIOR OVERLOADS
void SMSService::OnStart(DWORD argc, LPTSTR *argv) 
{
	// since service name is already know
	// we can initialize settings which implicitly use it
	if (!InitParameters())
		exit(1);

	InitGSM();

	if (!InitRPC())
		exit(2);
};
void SMSService::OnStop() 
{
};
void SMSService::OnShutdown() 
{
};
// end of BEHAVIOR OVERLOADS
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// REMOTE PROCEDURES
void SMSService::Post(SMS sms)
{
	try
	{
		fprintf(stderr, "SMS Post to %s\n", sms.Phone);
		return SMSService::m_gsm.Post(sms);
	}
	catch(PException e)
	{
		// log error
		fprintf(stderr, "exception: %s\n", e.text);
	}
}
int SMSService::Send(SMS sms)
{
	try
	{
		fprintf(stderr, "SMS Send to %s\n", sms.Phone);
		return SMSService::m_gsm.Send(sms);
	}
	catch(PException e)
	{
		// log error
		fprintf(stderr, "exception: %s\n", e.text);
	}
	return 0;
}
int SMSService::Receive(SMS *sms, int timeout)
{
	fprintf(stderr, "SMS Receive for %d ms\n", timeout);
	return m_gsm.Receive(sms, timeout);
}
// end of REMOTE PROCEDURES
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// DEBUG HELPERS
void SMSService::FormatErrorMessage(DWORD error, LPTSTR message, DWORD size)
{
	FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
		NULL, error, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
		message, size, NULL);
}
// end of DEBUG HELPERS
/////////////////////////////////////////////////////////////////////////////
